@@ -138,7 +138,7 @@ import Data.String (IsString)
 import BNFC.CF
 import BNFC.Backend.Base           (Backend, mkfile)
 import BNFC.Backend.Haskell.HsOpts
-import BNFC.Backend.Haskell.Utils  (parserName, catToType, mkDefName, typeToHaskell')
+import BNFC.Backend.Haskell.Utils  (parserName, catToType, mkDefName, typeToHaskell', comment)
 import BNFC.Options                (SharedOptions, TokenText(..), tokenText)
 import BNFC.PrettyPrint
 import BNFC.Utils                  (ModuleName, replace, when, table)
@@ -165,18 +165,18 @@ makeAgda
   -> Backend
 makeAgda time opts cf = do
   -- Generate AST bindings.
-  mkfile (agdaASTFile opts) $
+  mkfile (agdaASTFile opts) comment $
     cf2AgdaAST time (tokenText opts) (agdaASTFileM opts) (absFileM opts) (printerFileM opts) cf
   -- Generate parser bindings.
-  mkfile (agdaParserFile opts) $
+  mkfile (agdaParserFile opts) comment $
     cf2AgdaParser time (tokenText opts) (agdaParserFileM opts) (agdaASTFileM opts) (errFileM opts) (happyFileM opts)
       layoutMod
       parserCats
   -- Generate an I/O library for the test parser.
-  mkfile (agdaLibFile opts) $
+  mkfile (agdaLibFile opts) comment $
     agdaLibContents (agdaLibFileM opts)
   -- Generate test parser.
-  mkfile (agdaMainFile opts) $
+  mkfile (agdaMainFile opts) comment $
     agdaMainContents (agdaMainFileM opts) (agdaLibFileM opts) (agdaASTFileM opts) (agdaParserFileM opts)
       (hasLayout cf)
       (firstEntry cf)
@@ -315,7 +315,6 @@ preamble
   -> Doc
 preamble _time what = vcat $
   [ hcat [ "-- Agda bindings for the Haskell ", text what, "." ]
-  , hcat [ "-- Generated by BNFC." ]
   -- -- Time stamp does not work well with BNFC's mkfile logic.
   -- , hcat [ "-- Generated by BNFC at "         , text time, "." ]
   ]
@@ -656,7 +655,7 @@ incr = Map.alter $ maybe (Just 1) (Just . succ)
 definedRules :: CF -> Doc
 definedRules cf = vsep $ map mkDef $ definitions cf
   where
-    mkDef (Define f args e t) = vcat $ concat
+    mkDef (Define f args e _) = vcat $ concat
       [ [ text $ unwords [ mkDefName f, ":", typeToHaskell' "→" $ wpThing t ]
         | t <- maybeToList $ sigLookup f cf
         ]
@@ -863,8 +862,7 @@ agdaLibContents
   :: String      -- ^ Name of Agda library module.
   -> Doc         -- ^ Contents of Agda library module.
 agdaLibContents mod = vcat
-  [ "-- Generated by BNFC."
-  , "-- Basic I/O library."
+  [ "-- Basic I/O library."
   , ""
   , "module" <+> text mod <+> "where"
   , ""
@@ -930,7 +928,6 @@ agdaMainContents
   -> Doc         -- ^ Contents of Agda main module.
 agdaMainContents mod lmod amod pmod layout c = vcat
   [ "-- Test for Agda binding of parser.  Requires Agda >= 2.5.4."
-  , "-- Generated by BNFC."
   , ""
   , "module" <+> text mod <+> "where"
   , when layout "\nopen import Agda.Builtin.Bool using (true)"
