@@ -1,11 +1,12 @@
 {-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 {-
     BNF Converter: C Main file
     Copyright (C) 2004  Author:  Michael Pellauer
     Copyright (C) 2020  Andreas Abel
 -}
-module BNFC.Backend.C (makeC, bufferC, bufferH) where
+module BNFC.Backend.C (makeC, bufferC, bufferH, comment, testfileHeader) where
 
 import Prelude hiding ((<>))
 
@@ -28,24 +29,23 @@ import qualified BNFC.Backend.Common.Makefile as Makefile
 makeC :: SharedOptions -> CF -> MkFiles ()
 makeC opts cf = do
     let (hfile, cfile) = cf2CAbs (linenumbers opts) prefix cf
-    mkfile "Absyn.h" hfile
-    mkfile "Absyn.c" cfile
-    mkfile "Buffer.h" bufferH
-    mkfile "Buffer.c" $ bufferC "Buffer.h"
+    mkCFile "Absyn.h" hfile
+    mkCFile "Absyn.c" cfile
+    mkCFile "Buffer.h" bufferH
+    mkCFile "Buffer.c" $ bufferC "Buffer.h"
     let (flex, env) = cf2flex parserMode cf
-    mkfile (name ++ ".l") flex
-    let bison = cf2Bison (linenumbers opts) parserMode cf env
-    mkfile (name ++ ".y") bison
-    let header = mkHeaderFile (linenumbers opts) cf (Map.elems env)
-    mkfile "Parser.h" header
+    mkfile (name ++ ".l") commentWithEmacsModeHint flex
+    mkfile (name ++ ".y") commentWithEmacsModeHint $
+      cf2Bison (linenumbers opts) parserMode cf env
+    mkCFile "Parser.h" $ mkHeaderFile (linenumbers opts) cf (Map.elems env)
     let (skelH, skelC) = cf2CSkel cf
-    mkfile "Skeleton.h" skelH
-    mkfile "Skeleton.c" skelC
+    mkCFile "Skeleton.h" skelH
+    mkCFile "Skeleton.c" skelC
     let (prinH, prinC) = cf2CPrinter cf
-    mkfile "Printer.h" prinH
-    mkfile "Printer.c" prinC
-    mkfile "Test.c" (ctest cf)
-    Makefile.mkMakefile opts (makefile name prefix)
+    mkCFile "Printer.h" prinH
+    mkCFile "Printer.c" prinC
+    mkCFile "Test.c" (ctest cf)
+    Makefile.mkMakefile opts $ makefile name prefix
   where
     name :: String
     name = lang opts
@@ -56,6 +56,7 @@ makeC opts cf = do
     prefix = snakeCase_ name ++ "_"
     parserMode :: ParserMode
     parserMode = CParser False prefix
+    mkCFile x = mkfile x comment
 
 
 makefile :: String -> String -> String -> Doc
@@ -120,18 +121,30 @@ makefile name prefix basename = vcat
     ]
   where testName = "Test" ++ name
 
+-- | Put string into a block comment.
+comment :: String -> String
+comment x = unwords ["/*", x, "*/"]
+
+-- | C line comment including mode hint for emacs.
+commentWithEmacsModeHint :: String -> String
+commentWithEmacsModeHint = comment . ("-*- c -*- " ++)
+
+-- | A heading comment for the generated parser test.
+testfileHeader :: [String]
+testfileHeader =
+  [ "/************************* Compiler Front-End Test *************************/"
+  , "/*                                                                         */"
+  , "/*  This test will parse a file, print the abstract syntax tree, and then  */"
+  , "/*  pretty-print the result.                                               */"
+  , "/*                                                                         */"
+  , "/***************************************************************************/"
+  ]
+
 -- | Generate a test program that parses stdin and prints the AST and it's
 -- linearization
 ctest :: CF -> String
-ctest cf =
-  unlines
+ctest cf = unlines $ testfileHeader ++
    [
-    "/*** Compiler Front-End Test automatically generated by the BNF Converter ***/",
-    "/*                                                                          */",
-    "/* This test will parse a file, print the abstract syntax tree, and then    */",
-    "/* pretty-print the result.                                                 */",
-    "/*                                                                          */",
-    "/****************************************************************************/",
     "",
     "#include <stdio.h>",
     "#include <stdlib.h>",
@@ -206,10 +219,11 @@ ctest cf =
   dat = identCat . normCat $ cat
 
 mkHeaderFile :: RecordPositions -> CF -> [String] -> String
-mkHeaderFile _ cf env = unlines $ concat
+mkHeaderFile _ cf _env = unlines $ concat
   [ [ "#ifndef PARSER_HEADER_FILE"
     , "#define PARSER_HEADER_FILE"
     , ""
+    , "#include <stdio.h>"
     , "#include \"Absyn.h\""
     , ""
     ]
@@ -232,9 +246,7 @@ mkHeaderFile _ cf env = unlines $ concat
 
 bufferH :: String
 bufferH = unlines
-  [ "/* This utility file was automatically generated by BNFC. */"
-  , ""
-  , "/* A dynamically allocated character buffer that grows as it is appended. */"
+  [ "/* A dynamically allocated character buffer that grows as it is appended. */"
   , ""
   , "#ifndef BUFFER_HEADER"
   , "#define BUFFER_HEADER"
@@ -276,9 +288,7 @@ bufferH = unlines
 
 bufferC :: String -> String
 bufferC bufferH = unlines
-  [ "/* This utility file was automatically generated by BNFC. */"
-  , ""
-  , "/* A dynamically allocated character buffer that grows as it is appended. */"
+  [ "/* A dynamically allocated character buffer that grows as it is appended. */"
   , ""
   , "#include <assert.h>  /* assert */"
   , "#include <stdlib.h>  /* free, malloc */"
