@@ -26,11 +26,10 @@ import Data.Char      ( toLower )
 import Data.Either    ( lefts )
 import Data.Foldable  ( toList )
 import Data.List      ( nub )
-import Data.Maybe     ( isJust )
 
 import BNFC.CF
 import BNFC.PrettyPrint
-import BNFC.Utils     ( (+++), uniqOn, whenJust, unless, unlessNull )
+import BNFC.Utils     ( (+++), uniqOn, unless, unlessNull )
 
 import BNFC.Backend.Common
 import BNFC.Backend.Common.NamedVariables
@@ -153,7 +152,7 @@ mkCFile cf groups = concat
     eps = nub . map normCat . toList $ allEntryPoints cf
     header = unlines
      [
-      "/*** BNFC-Generated Pretty Printer and Abstract Syntax Viewer ***/",
+      "/*** Pretty Printer and Abstract Syntax Viewer ***/",
       "",
       "#include <ctype.h>   /* isspace */",
       "#include <stdio.h>",
