@@ -23,10 +23,11 @@ module BNFC.Backend.CPP.PrettyPrinter (cf2CPPPrinter, prRender) where
 
 import Prelude hiding ((<>))
 
-import Data.Char(toLower)
+import Data.Bifunctor (second)
+import Data.Char  (toLower)
 
 import BNFC.CF
-import BNFC.Utils ((+++), when)
+import BNFC.Utils
 import BNFC.Backend.Common
 import BNFC.Backend.Common.NamedVariables
 import BNFC.Backend.Common.StrUtils (renderCharOrString)
@@ -89,13 +90,16 @@ mkHFile useStl inPackage cf groups = unlines
     "  void render(const char *s);",
     "  void indent(void);",
     "  void backup(void);",
+    "  void onEmptyLine(void);",
+    "  void removeTrailingSpaces(void);",
+    "  void removeTrailingWhitespace(void);",
     " public:",
     "  PrintAbsyn(void);",
     "  ~PrintAbsyn(void);",
     "  char *print(Visitable *v);"
    ]
   hdef = nsDefine inPackage "PRINTER_HEADER"
-  content = concatMap prDataH groups
+  content = concatMap (prDataH useStl) groups
   classFooter = unlines $
    [
     "  void visitInteger(Integer i);",
@@ -189,14 +193,18 @@ mkHFile useStl inPackage cf groups = unlines
    ]
 
 --Prints all the required method names and their parameters.
-prDataH :: (Cat, [Rule]) -> String
-prDataH (cat, rules) =
- if isList cat
- then concat ["  void visit", cl, "(", cl, " *p);\n"]
- else abstract ++ concatMap prRuleH rules
+prDataH :: Bool -> (Cat, [Rule]) -> String
+prDataH useSTL (cat, rules)
+ | isList cat = unlines $ concat
+     [ [ concat [ "  void visit", cl, "(", cl, " *p);"          ] ]
+     , when useSTL
+       [ concat [ "  void iter", cl, "(", itty, " i, ", itty, " j);" ] ]
+     ]
+ | otherwise  = abstract ++ concatMap prRuleH rules
  where
-   cl = identCat (normCat cat)
-   abstract = case lookupRule (noPosition $ show cat) rules of
+   cl       = identCat (normCat cat)
+   itty     = concat [ cl, "::", "const_iterator" ]
+   abstract = case lookupRule (noPosition $ catToStr cat) rules of
     Just _ -> ""
     Nothing ->  "  void visit" ++ cl ++ "(" ++ cl ++ " *p); /* abstract class */\n"
 
@@ -228,7 +236,7 @@ mkCFile useStl inPackage cf groups = concat
   where
     header = unlines
      [
-      "/*** BNFC-Generated Pretty Printer and Abstract Syntax Viewer ***/",
+      "/*** Pretty Printer and Abstract Syntax Viewer ***/",
       "",
       "#include <string>",
       "#include \"Printer.H\"",
@@ -394,140 +402,151 @@ prPrintData _ inPackage _cf (cat, rules) = -- Not a list
     abstract ++ concatMap (prPrintRule inPackage) rules
   where
   cl = identCat (normCat cat)
-  abstract = case lookupRule (noPosition $ show cat) rules of
+  abstract = case lookupRule (noPosition $ catToStr cat) rules of
     Just _ -> ""
     Nothing ->  "void PrintAbsyn::visit" ++ cl ++ "(" ++ cl +++ "*p) {} //abstract class\n\n"
 
--- | Generate pretty printer visitor for a list category:
+-- | Generate pretty printer visitor for a list category (STL version).
 --
--- >>> let c = Cat "C" ; lc = ListCat c
--- >>> let rules = [npRule "[]" lc [] Parsable, npRule "(:)" lc [Left c, Right "-", Left lc] Parsable]
--- >>> genPrintVisitorList (lc, rules)
--- void PrintAbsyn::visitListC(ListC *listc)
--- {
---   for (ListC::const_iterator i = listc->begin() ; i != listc->end() ; ++i)
---   {
---     (*i)->accept(this);
---     render('-');
---   }
--- }
---
--- >>> let c2 = CoercCat "C" 2 ; lc2 = ListCat c2
--- >>> let rules2 = rules ++ [npRule "[]" lc2 [] Parsable, npRule "(:)" lc2 [Left c2, Right "+", Left lc2] Parsable]
--- >>> genPrintVisitorList (lc, rules2)
--- void PrintAbsyn::visitListC(ListC *listc)
--- {
---   for (ListC::const_iterator i = listc->begin() ; i != listc->end() ; ++i)
---   {
---     (*i)->accept(this);
---     switch(_i_)
---     {
---       case 2: render('+'); break;
---       default: render('-');
---     }
---   }
--- }
 genPrintVisitorList :: (Cat, [Rule]) -> Doc
-genPrintVisitorList (cat@(ListCat c), rules) =
-    "void PrintAbsyn::visit" <> text cl <> "(" <> text cl <> " *" <> vname <> ")"
-    $$ codeblock 2
-      [ "for ("<> text cl <> "::const_iterator i = " <> vname <> "->begin() ; i != " <> vname <> "->end() ; ++i)"
-      , codeblock 2
-          [ if isTokenCat c
-              then "visit" <> text (baseName cl) <> "(*i) ;"
-              else "(*i)->accept(this);"
-          , (if hasOneFunc rules
-              then "if (i != " <> vname <> "->end() - 1)"
-              else empty)
-            <+> renderListSepByPrecedence "_i_" renderSep separators
-          ]
+genPrintVisitorList (cat@(ListCat _), rules) = vcat
+  [ "void PrintAbsyn::visit" <> lty <> parens (lty <+> "*" <> vname)
+  , codeblock 2
+    [ "iter" <> lty <> parens (vname <> "->begin()" <> comma <+> vname <> "->end()") <> semi ]
+  , ""
+  , "void PrintAbsyn::iter" <> lty <> parens (itty <+> "i" <> comma <+> itty <+> "j")
+  , codeblock 2 $ concat
+    [ if null docs0 then
+      [ "if (i == j) return;" ]
+      else
+      [ "if (i == j)"
+      , "{ /* nil */"
+      , nest 2 $ vcat docs0
+      , "}"
+      , "else"
       ]
+    , unless (null docs1)
+      [ "if (i == j-1)"
+      , "{ /* last */"
+      , nest 2 $ vcat docs1
+      , "}"
+      , "else"
+      ]
+    , [ "{ /* cons */"
+      ,  nest 2 $ vcat docs2
+      , "}"
+      ]
+    ]
+  , ""
+  , ""
+  ]
   where
-   separators  = getSeparatorByPrecedence rules
-   cl          = identCat (normCat cat)
-   vname       = text $ map toLower cl
-   renderSep s = "render(" <> text (snd (renderCharOrString s)) <> ")"
+  cl        = identCat (normCat cat)
+  lty       = text cl                   -- List type
+  itty      = lty <> "::const_iterator" -- Iterator type
+  vname     = text $ map toLower cl
+  prules    = sortRulesByPrecedence rules
+  swRules f = switchByPrecedence "_i_" $
+                map (second $ sep . prListRule_) $
+                  uniqOn fst $ filter f prules
+                  -- Discard duplicates, can only handle one rule per precedence.
+  docs0     = swRules isNilFun
+  docs1     = swRules isOneFun
+  docs2     = swRules isConsFun
 
 genPrintVisitorList _ = error "genPrintVisitorList expects a ListCat"
 
--- | This is the only part of the pretty printer that differs significantly
+-- | Only render the rhs (items) of a list rule.
+
+prListRule_ :: IsFun a => Rul a -> [Doc]
+prListRule_ (Rule _ _ items _) = for items $ \case
+  Right t       -> "render(" <> text (snd (renderCharOrString t)) <> ");"
+  Left c
+    | Just{} <- maybeTokenCat c
+                -> "visit" <> dat <> "(*i);"
+    | isList c  -> "iter" <> dat <> "(i+1, j);"
+    | otherwise -> "(*i)->accept(this);"
+    where
+    dat = text $ identCat $ normCat c
+
+-- This is the only part of the pretty printer that differs significantly
 -- between the versions with and without STL.
+-- The present version has been adapted from CFtoCPrinter.
 genPrintVisitorListNoStl :: (Cat, [Rule]) -> String
-genPrintVisitorListNoStl (cat@(ListCat c), rules) = unlines $ concat
+genPrintVisitorListNoStl (cat@(ListCat _), rules) = unlines $ concat
   [ [ "void PrintAbsyn::visit" ++ cl ++ "("++ cl ++ " *" ++ vname ++ ")"
     , "{"
-    , "  while(" ++ vname +++ "!= 0)"
-    , "  {"
-    , "    if (" ++ vname ++ "->" ++ vname ++ "_ == 0)"
-    , "    {"
-    , visitMember
+      , "  if (" ++ vname +++ "== 0)"
+      , "  { /* nil */"
+      ]
+    , unlessNull (swRules isNilFun) $ \ docs ->
+      [ render $ nest 4 $ vcat docs ]
+    , [ "  }" ]
+    , unlessNull (swRules isOneFun) $ \ docs ->
+      [ "  else if (" ++ pre ++ vname ++ "_ == 0)"
+      , "  { /* last */"
+      , render $ nest 4 $ vcat docs
+      , "  }"
+      ]
+    , unlessNull (swRules isConsFun) $ \ docs ->
+      [ "  else"
+      , "  { /* cons */"
+      , render $ nest 4 $ vcat docs
+      , "  }"
+      ]
+    , [ "}"
+      , ""
+      ]
     ]
-  , optsep
-  , [ "      " ++ vname +++ "= 0;"
-    , "    }"
-    , "    else"
-    , "    {"
-    , visitMember
-    , render $ nest 6 $ renderListSepByPrecedence "_i_" renderSep separators
-    , "      " ++ vname +++ "=" +++ vname ++ "->" ++ vname ++ "_;"
-    , "    }"
-    , "  }"
+  where
+  cl          = identCat (normCat cat)
+  vname       = map toLower cl
+  pre         = vname ++ "->"
+  prules      = sortRulesByPrecedence rules
+  swRules f   = switchByPrecedence "_i_" $
+                  map (second $ sep . map text . prPrintRule_ pre) $
+                    uniqOn fst $ filter f prules
+                    -- Discard duplicates, can only handle one rule per precedence.
+genPrintVisitorListNoStl _ = error "genPrintVisitorListNoStl expects a ListCat"
+
+--Pretty Printer methods for a rule.
+prPrintRule :: Maybe String -> Rule -> String
+prPrintRule inPackage r@(Rule fun _ _ _) | isProperLabel fun = unlines $ concat
+  [ [ "void PrintAbsyn::visit" ++ funName fun ++ "(" ++ funName fun +++ "*" ++ fnm ++ ")"
+    , "{"
+    , "  int oldi = _i_;"
+    , parenCode "_L_PAREN"
+    , ""
+    ]
+  , prPrintRule_ (fnm ++ "->") r
+  , [ ""
+    , parenCode "_R_PAREN"
+    , "  _i_ = oldi;"
     , "}"
     , ""
     ]
   ]
   where
-    visitMember
-      | Just t <- maybeTokenCat c =
-          "      visit" ++ t ++ "(" ++ vname ++ "->" ++ member ++ ");"
-      | otherwise =
-          "      " ++ vname ++ "->" ++ member ++ "->accept(this);"
-    cl     = identCat (normCat cat)
-    ecl    = identCat (normCatOfList cat)
-    vname  = map toLower cl
-    member = map toLower ecl ++ "_"
-    optsep = if hasOneFunc rules || null sep' then []
-             else [ "      render(" ++ sep' ++ ");" ]
-    sep' = snd $ renderCharOrString $ getCons rules
-    renderSep s = "render(" <> text (snd $ renderCharOrString s) <> ")"
-    separators = getSeparatorByPrecedence rules
-genPrintVisitorListNoStl _ = error "genPrintVisitorListNoStl expects a ListCat"
-
---Pretty Printer methods for a rule.
-prPrintRule :: Maybe String -> Rule -> String
-prPrintRule inPackage r@(Rule fun _ cats _) | isProperLabel fun = unlines
-  [
-   "void PrintAbsyn::visit" ++ funName fun ++ "(" ++ funName fun +++ "*" ++ fnm ++ ")",
-   "{",
-   "  int oldi = _i_;",
-   lparen,
-   cats',
-   rparen,
-   "  _i_ = oldi;",
-   "}",
-   ""
-  ]
-   where
-    p = precRule r
-    (lparen, rparen) =
-      ("  if (oldi > " ++ show p ++ ") render(" ++ nsDefine inPackage "_L_PAREN" ++ ");\n",
-       "  if (oldi > " ++ show p ++ ") render(" ++ nsDefine inPackage "_R_PAREN" ++ ");\n")
-    cats' = concatMap (prPrintCat fnm) (numVars cats)
-    fnm = "p" --old names could cause conflicts
+  p = precRule r
+  parenCode x = "  if (oldi > " ++ show p ++ ") render(" ++ nsDefine inPackage x ++ ");"
+  fnm = "p" --old names could cause conflicts
 prPrintRule _ _ = ""
 
+prPrintRule_ :: IsFun a => String -> Rul a -> [String]
+prPrintRule_ pre (Rule _ _ items _) = map (prPrintItem pre) $ numVars items
+
 --This goes on to recurse to the instance variables.
-prPrintCat :: String -> Either (Cat, Doc) String -> String
-prPrintCat _ (Right t) = "  render(" ++ t' ++ ");\n"
-  where t' = snd (renderCharOrString t)
-prPrintCat fnm (Left (c, nt))
+prPrintItem :: String -> Either (Cat, Doc) String -> String
+prPrintItem _   (Right t) = "  render(" ++ snd (renderCharOrString t) ++ ");"
+prPrintItem pre (Left (c, nt))
   | Just t <- maybeTokenCat c
-              = "  visit" ++ t ++ "(" ++ fnm ++ "->" ++ s ++ ");\n"
-  | isList c  = "  if(" ++ fnm ++ "->" ++ s ++ ") {" ++ accept ++ "}\n"
-  | otherwise = "  " ++ accept ++ "\n"
+              = "  visit" ++ t   ++ "(" ++ pre ++ s ++ ");"
+  | isList c  = "  " ++ setI (precCat c) ++
+                  "visit" ++ elt ++ "(" ++ pre ++ s ++ ");"
+  | otherwise = "  " ++ setI (precCat c) ++ pre ++ s ++ "->accept(this);"
   where
-    s = render nt
-    accept = setI (precCat c) ++ fnm ++ "->" ++ s ++ "->accept(this);"
+  s   = render nt
+  elt = identCat $ normCat c
 
 {- **** Abstract Syntax Tree Printer **** -}
 
@@ -587,7 +606,7 @@ prShowData _ (cat, rules) =  --Not a list:
   abstract ++ concatMap prShowRule rules
   where
     cl = identCat (normCat cat)
-    abstract = case lookupRule (noPosition $ show cat) rules of
+    abstract = case lookupRule (noPosition $ catToStr cat) rules of
       Just _ -> ""
       Nothing ->  "void ShowAbsyn::visit" ++ cl ++ "(" ++ cl ++ " *p) {} //abstract class\n\n"
 
@@ -660,8 +679,7 @@ prRender useStl = unlines $ concat
       "{",
       "  if (c == '{')",
       "  {",
-      "     bufAppend('\\n');",
-      "     indent();",
+      "     onEmptyLine();",
       "     bufAppend(c);",
       "     _n_ = _n_ + INDENT_WIDTH;",
       "     bufAppend('\\n');",
@@ -671,30 +689,27 @@ prRender useStl = unlines $ concat
       "     bufAppend(c);",
       "  else if (c == ')' || c == ']')",
       "  {",
-      "     backup();",
+      "     removeTrailingWhitespace();",
       "     bufAppend(c);",
       "     bufAppend(' ');",
       "  }",
       "  else if (c == '}')",
       "  {",
-      "     int t;",
       "     _n_ = _n_ - INDENT_WIDTH;",
-      "     for (t=0; t<INDENT_WIDTH; t++) {",
-      "       backup();",
-      "     }",
+      "     onEmptyLine();",
       "     bufAppend(c);",
       "     bufAppend('\\n\');",
       "     indent();",
       "  }",
       "  else if (c == ',')",
       "  {",
-      "     backup();",
+      "     removeTrailingWhitespace();",
       "     bufAppend(c);",
       "     bufAppend(' ');",
       "  }",
       "  else if (c == ';')",
       "  {",
-      "     backup();",
+      "     removeTrailingWhitespace();",
       "     bufAppend(c);",
       "     bufAppend('\\n');",
       "     indent();",
@@ -750,9 +765,28 @@ prRender useStl = unlines $ concat
     , ""
     , "void PrintAbsyn::backup()"
     , "{"
-    , "  if (buf_[cur_ - 1] == ' ')"
+    , "  if (cur_ && buf_[cur_ - 1] == ' ')"
     , "    buf_[--cur_] = 0;"
     , "}"
     , ""
+    , "void PrintAbsyn::removeTrailingSpaces()"
+    , "{"
+    , "  while (cur_ && buf_[cur_ - 1] == ' ') --cur_;"
+    , "  buf_[cur_] = 0;"
+    , "}"
+    , ""
+    , "void PrintAbsyn::removeTrailingWhitespace()"
+    , "{"
+    , "  while (cur_ && (buf_[cur_ - 1] == ' ' || buf_[cur_ - 1] == '\\n')) --cur_;"
+    , "  buf_[cur_] = 0;"
+    , "}"
+    , ""
+    , "void PrintAbsyn::onEmptyLine()"
+    , "{"
+    , "  removeTrailingSpaces();"
+    , "  if (cur_ && buf_[cur_ - 1 ] != '\\n') bufAppend('\\n');"
+    , "  indent();"
+    , "}"
+    , ""
     ]
   ]
